@@ -95,7 +95,7 @@ def test_topology_total_degree(
     """
     logging.info("Evaluating the test topology total degree, please wait")
     network = rc.ReadTsv(network_file).get_network()
-
+    data = rc.ReadTxt(geneset_file).get_data()
     geneset = rc.ReadGmt(geneset_file).get_geneset(setname)
 
     setnames = [key for key in geneset.keys()]
@@ -106,7 +106,7 @@ def test_topology_total_degree(
 
     # Create table
     output1.create_st_table_empirical()
-    st_test = st.StatisticalTest(st.geneset_total_degree_statistic, network)
+    st_test = st.StatisticalTest(st.geneset_total_degree_statistic, network, degree_bins = n_bins)
 
     for setname, item in geneset.items():
         # Geneset smaller than size cut are not taken into consideration
@@ -142,10 +142,11 @@ def test_topology_internal_degree(
     size_cut: "removes all genesets with a mapped length < size_cut" = 20,
     number_of_permutations: "number of permutations for computing the empirical pvalue" = 500,
     cores: "Number of cores for the multiprocessing" = 1,
+    n_bins: 'if >1 applies degree correction by binning the node degrees and sampling according to geneset distribution' = 1,
     results_figure: "barplot of results, use pdf or png extension" = None,
     diagnostic_null_folder: "plot null distribution, pass the folder where all the figures are going to be saved "
                             "(one for each dataset)" = None,
-):
+    ):
     """
     Performs the analysis of internal degree.
     It computes a p-value for the ratio of internal degree of the geneset being bigger than the one expected by chance
@@ -158,7 +159,7 @@ def test_topology_internal_degree(
     output1 = out.Output(network_file, output_table, "topology_internal_degree", geneset_file, setnames)
     logging.info("Results file = " + output1.output_table_results)
     output1.create_st_table_empirical()
-    st_test = st.StatisticalTest(st.geneset_internal_degree_statistic, network)
+    st_test = st.StatisticalTest(st.geneset_internal_degree_statistic, network, degree_bins = n_bins)
     for setname, item in geneset.items():
         item = set(item)
         if len(item) > size_cut:
@@ -192,10 +193,11 @@ def test_topology_rwr(
     number_of_permutations: "number of permutations for computing the empirical pvalue" = 500,
     cores: "Number of cores for the multiprocessing" = 1,
     in_memory: "set if you want the large matrix to be read in memory" = False,
+    n_bins: 'if >1 applies degree correction by binning the node degrees and sampling according to geneset distribution' = 1,
     results_figure: "barplot of results, use pdf or png extension" = None,
     diagnostic_null_folder: "plot null distribution, pass the folder where all the figures are going to be saved "
                             "(one for each dataset)" = None,
-):
+ ):
     """
     Performs the analysis of random walk probabilities.
     Given the RWR matrix, it compares the probability of walking between the genes in the geneset compared to
@@ -213,7 +215,7 @@ def test_topology_rwr(
 
     logging.info("Results file = " + output1.output_table_results)
     output1.create_st_table_empirical()
-    st_test = st.StatisticalTest(st.geneset_RW_statistic, network, rw_dict)
+    st_test = st.StatisticalTest(st.geneset_RW_statistic, network, rw_dict, degree_bins = n_bins)
 
     for setname, item in geneset.items():
         item = set(item)
@@ -249,11 +251,12 @@ def test_topology_module(
     size_cut: "removes all genesets with a mapped length < size_cut" = 20,
     number_of_permutations: "number of permutations for computing the empirical pvalue" = 500,
     cores: "Number of cores for the multiprocessing" = 1,
+    n_bins: 'if >1 applies degree correction by binning the node degrees and sampling according to geneset distribution' = 1,
     output_lcc: "for creating a GMT file with the LCC lists pass a GMT filename" = None,
     results_figure: "barplot of results, use pdf or png extension" = None,
     diagnostic_null_folder: "plot null distribution, pass the folder where all the figures are going to be saved "
                             "(one for each dataset)" = None,
-):
+ ):
     """
     Performs geneset network topology module analysis.
     It computes a p-value for the largest connected component of the geneset being bigger than the one expected by chance
@@ -267,7 +270,7 @@ def test_topology_module(
     logging.info("Results file = " + output1.output_table_results)
     output1.create_st_table_empirical()
 
-    st_test = st.StatisticalTest(st.geneset_module_statistic, network)
+    st_test = st.StatisticalTest(st.geneset_module_statistic, network, degree_bins = n_bins)
     for setname, item in geneset.items():
         item = set(item)
         if len(item) > size_cut:
@@ -310,6 +313,7 @@ def test_topology_sp(
     number_of_permutations: "number of permutations for computing the empirical pvalue" = 500,
     cores: "Number of cores for the multiprocessing" = 1,
     in_memory: "set if you want the large matrix to be read in memory" = False,
+    n_bins: 'if >1 applies degree correction by binning the node degrees and sampling according to geneset distribution' = 1,
     results_figure: "barplot of results, use pdf or png extension" = None,
     diagnostic_null_folder: "plot null distribution, pass the folder where all the figures are going to be saved "
                             "(one for each dataset)" = None,
@@ -337,7 +341,7 @@ def test_topology_sp(
     output1 = out.Output(network_file, output_table, "topology_sp", geneset_file, setnames)
     logging.info("Results file = " + output1.output_table_results)
     output1.create_st_table_empirical()
-    st_test = st.StatisticalTest(st.geneset_localisation_statistic, network, diz)
+    st_test = st.StatisticalTest(st.geneset_localisation_statistic, network, diz, degree_bins = n_bins)
 
     for setname, item in geneset.items():
 
@@ -452,6 +456,7 @@ def test_association_sp(
     cores: "Number of cores for the multiprocessing" = 1,
     in_memory: "set if you want the large matrix to be read in memory" = False,
     number_of_permutations: "number of permutations for computing the empirical pvalue" = 500,
+    n_bins: 'if >1 applies degree correction by binning the node degrees and sampling according to geneset distribution' = 1,
     results_figure: "barplot of results, use pdf or png extension" = None,
 ):
     """
@@ -476,7 +481,7 @@ def test_association_sp(
     # Read matrix
     sp_diz = {"nodes": read_distance_matrix(distance_matrix_filename, in_memory=in_memory)[0],
               "matrix": read_distance_matrix(distance_matrix_filename, in_memory=in_memory)[1]}
-    # TODO check why "in memory"= True results are different from "in memory"= False
+
     sp_diz["matrix"] = sp_diz["matrix"] + np.transpose(sp_diz["matrix"])
     np.fill_diagonal(sp_diz["matrix"], np.inf)
 
@@ -493,7 +498,7 @@ def test_association_sp(
         else:
             geneset_b = None
 
-    st_comparison = sc.StatisticalComparison(sc.comparison_shortest_path, network, diz=sp_diz, n_proc=cores)
+    st_comparison = sc.StatisticalComparison(sc.comparison_shortest_path, network, diz=sp_diz, n_proc=cores, degree_bins = n_bins)
 
     if not geneset_b:  # Analysis of genesets inside a single file
         logging.info("Analysing all the sets in " + file_geneset_a)
@@ -562,6 +567,7 @@ def test_association_rwr(
     cores: "Number of cores for the multiprocessing" = 1,
     in_memory: "set if you want the large matrix to be read in memory" = False,
     number_of_permutations: "number of permutations for computing the empirical pvalue" = 500,
+    n_bins: 'if >1 applies degree correction by binning the node degrees and sampling according to geneset distribution' = 1,
     results_figure: "heatmap of results" = None,
 ):
     """
@@ -600,7 +606,7 @@ def test_association_rwr(
         else:
             geneset_b = None
 
-    st_comparison = sc.StatisticalComparison(sc.comparison_random_walk, network, n_proc=cores, diz=rw_dict)
+    st_comparison = sc.StatisticalComparison(sc.comparison_random_walk, network, n_proc=cores, diz=rw_dict, degree_bins = n_bins)
     if not geneset_b:
         logging.info("Analysing all the sets in " + file_geneset_a)
         setnames = [key for key in geneset_a.keys()]
