@@ -4,6 +4,7 @@ import numpy as np
 import sys
 import multiprocessing
 import pygna.statistical_test as st
+from pygna.utils import get_sampling_p, get_node_bins_map
 
 class StatisticalComparison:
     """
@@ -11,7 +12,7 @@ class StatisticalComparison:
     Please refer to the single method documentation for the returning values
     """
 
-    def __init__(self, comparison_statistic, network: nx.Graph, n_proc: int = 1, diz: dict = {}):
+    def __init__(self, comparison_statistic, network: nx.Graph, n_proc: int = 1, diz: dict = {}, degree_bins = 1):
         """
         :param comparison_statistic: the method do be applied to perform the statistical analysis
         :param network: the network to be used during the analysis
@@ -22,18 +23,38 @@ class StatisticalComparison:
         self.__network = network
         self.__diz = diz
         self.__n_proc = n_proc
+        self.degree_bins = degree_bins
+        self.node_2_bin_map = None
+        self.bins = None
 
         if (type(self.__network) is nx.Graph) or (type(self.__network) is nx.DiGraph):
-            self.__universe = set(self.__network.nodes())
+            self.__universe = list(set(self.__network.nodes()))
         elif type(self.__network) is dict:
-            self.__universe = set(self.__network.keys())
+            self.__universe = list(set(self.__network.keys()))
 
         else:
             logging.error("Unknown network type: %s" % type(self.__network))
             sys.exit(-1)
 
+        # If number of bins is specified, the mapping is generated
+        if (degree_bins>1):
+
+            if type(self.__network) is dict:
+                logging.error('Cannot do degree corrected sampling without the whole network')
+
+            degree= list(self.__network.degree(nbunch = self.__universe))
+            [nn,dd] = list(zip(*degree))
+
+            # We make sure that later the universe corresponds to the mapping
+            self.__universe = list(nn)
+            histogram, bins_edge = np.histogram(list(dd), bins = degree_bins, density = False)
+            self.bins = {int(i):{'nk':histogram[i], 'range':(bins_edge[i],bins_edge[i+1])} for i in range(len(histogram))}
+            self.bins[len(self.bins.keys())-1]['range'] = (self.bins[len(self.bins.keys())-1]['range'][0], self.bins[len(self.bins.keys())-1]['range'][1]+1)
+            self.node_2_bin_map = get_node_bins_map(degree, self.bins)
+
     def comparison_empirical_pvalue(self, genesetA: set, genesetB: set, alternative: str = "less", max_iter: int = 100,
                                     keep: bool = False) -> [int, float, float, int, int]:
+        
         """
         Calculate the empirical value between two genesets
 
@@ -46,27 +67,43 @@ class StatisticalComparison:
         """
 
         # mapping genesets
-        mapped_genesetA = sorted(list(set(genesetA) & self.__universe))
-        mapped_genesetB = sorted(list(set(genesetB) & self.__universe))
+        mapped_genesetA = sorted(list(set(genesetA) & set(self.__universe)))
+        mapped_genesetB = sorted(list(set(genesetB) & set(self.__universe)))
         logging.info("Mapped %d genes out of %d from genesetA" % (len(mapped_genesetA), len(genesetA)))
         logging.info("Mapped %d genes out of %d from genesetB" % (len(mapped_genesetB), len(genesetB)))
-
-        # Observed statistical values
-        observed = self.__comparison_statistic(self.__network, mapped_genesetA, mapped_genesetB, self.__diz)
-
-        # iterations
-        null_distribution = StatisticalComparison.get_comparison_null_distribution_mp(self, mapped_genesetA,
-                                                                                      mapped_genesetB, max_iter, keep)
-        pvalue = 1
-        if alternative == "greater":
-            pvalue = (np.sum(null_distribution >= observed) + 1) / (float(len(null_distribution) + 1))
+        if (len(mapped_genesetA) == 0) or (len(mapped_genesetB) == 0):
+            return 0, 1, np.array([0]), 0, 0
         else:
-            pvalue = (np.sum(null_distribution <= observed) + 1) / (float(len(null_distribution) + 1))
 
-        return observed, pvalue, null_distribution, len(mapped_genesetA), len(mapped_genesetB)
+            sampling_p_a = None
+            sampling_p_b = None
+            if self.degree_bins > 1:
+                # if more than one bin is specified, the sampling probability is generated
+                sampling_p_a = get_sampling_p(mapped_genesetA, self.__network, self.bins, self.node_2_bin_map)
+                if (~keep):
+                    sampling_p_b = get_sampling_p(mapped_genesetB, self.__network, self.bins, self.node_2_bin_map)
+
+            # Observed statistical values
+            observed = self.__comparison_statistic(self.__network, mapped_genesetA, mapped_genesetB, self.__diz)
+
+            # iterations
+            null_distribution = StatisticalComparison.get_comparison_null_distribution_mp(self, mapped_genesetA,
+                                                                                          mapped_genesetB, max_iter, keep,
+                                                                                          sampling_p_a = sampling_p_a,
+                                                                                          sampling_p_b = sampling_p_b,
+                                                                                          )
+            pvalue = 1
+            if alternative == "greater":
+                pvalue = (np.sum(null_distribution >= observed) + 1) / (float(len(null_distribution) + 1))
+            else:
+                pvalue = (np.sum(null_distribution <= observed) + 1) / (float(len(null_distribution) + 1))
+
+            return observed, pvalue, null_distribution, len(mapped_genesetA), len(mapped_genesetB)
 
     def get_comparison_null_distribution_mp(self, genesetA: list, genesetB: list, max_iter: int = 100,
-                                            keep: bool = False) -> np.ndarray:
+                                            keep: bool = False, 
+                                            sampling_p_a = None, 
+                                            sampling_p_b = None) -> np.ndarray:
         """
         Calculate the null distribution between two genesets with multiple CPUs
 
@@ -74,6 +111,8 @@ class StatisticalComparison:
         :param genesetB: the second geneset to compare
         :param max_iter: maximum number of iteration to perform
         :param keep: if the geneset B should not be kept
+        :param sampling_p_a: random sampling probability for geneset a
+        :param sampling_p_b: random sampling probability for geneset b
         :return: the array with null distribution
         """
 
@@ -83,11 +122,13 @@ class StatisticalComparison:
         # Don't set up the multicore architecture if only one core is given
         if self.__n_proc == 1:
             null_distribution = StatisticalComparison.get_comparison_null_distribution(self, genesetA, genesetB,
-                                                                                       max_iter, keep)
+                                                                                       max_iter, keep,
+                                                                                       sampling_p_a = sampling_p_a, 
+                                                                                       sampling_p_b = sampling_p_b)
         else:
             p = multiprocessing.Pool(self.__n_proc)
             results = [p.apply_async(StatisticalComparison.get_comparison_null_distribution,
-                                     args=(self, genesetA, genesetB, n_trial, keep)) for w in
+                                     args=(self, genesetA, genesetB, n_trial, keep, sampling_p_a, sampling_p_b)) for w in
                        list(range(1, self.__n_proc + 1))]
 
             null_distribution = np.array([])
@@ -97,7 +138,9 @@ class StatisticalComparison:
 
         return np.asarray(null_distribution)
 
-    def get_comparison_null_distribution(self, genesetA: list, genesetB: list, n_samples: int, keep: bool) -> list:
+    def get_comparison_null_distribution(self, genesetA: list, genesetB: list, n_samples: int, keep: bool, 
+                                            sampling_p_a = None, 
+                                            sampling_p_b = None) -> list:
         """
         Calculate the null distribution between two genesets with single CPU
 
@@ -113,14 +156,14 @@ class StatisticalComparison:
         # association statistic, B kept, A bootstrapped
         if keep:
             for i in range(n_samples):
-                random_sample_A = np.random.choice(list(self.__universe), len(genesetA), replace=False)
+                random_sample_A = np.random.choice(self.__universe, len(genesetA), replace=False, p = sampling_p_a)
                 random_dist.append(self.__comparison_statistic(self.__network, set(random_sample_A), set(genesetB),
                                                                self.__diz))
         # association statistic, B kept, A bootstrapped
         else:
             for i in range(n_samples):
-                random_sample_A = np.random.choice(list(self.__universe), len(genesetA), replace=False)
-                random_sample_B = np.random.choice(list(self.__universe), len(genesetB), replace=False)
+                random_sample_A = np.random.choice(self.__universe, len(genesetA), replace=False, p = sampling_p_a)
+                random_sample_B = np.random.choice(self.__universe, len(genesetB), replace=False, p = sampling_p_b)
                 random_dist.append(self.__comparison_statistic(self.__network, set(random_sample_A),
                                                                set(random_sample_B), self.__diz))
         return random_dist
@@ -163,7 +206,7 @@ def calculate_sum(n: np.ndarray, m: np.ndarray, diz: dict) -> np.ndarray:
     diz = diz["matrix"]
     sub_matrix = diz[n[:, None], m]
     sub_matrix = np.where(sub_matrix != np.inf, sub_matrix, 0)
-    min_columns = np.amin(sub_matrix, axis=0)
+    min_columns = np.amin(sub_matrix, axis=0)   
     sum_columns = np.sum(min_columns)
     return sum_columns
 
